// Generated by CoffeeScript 1.3.3
(function() {
  var Match, NoMatch, Parser, ParserState, check, drop, end, foldLeft, implicit, newState, optional, parse, regex, reject, repeat, resolve, seq, setWhitespace, string, times, whitespace, _parser_id,
    __slice = [].slice;

  newState = function(text) {
    return new ParserState(text, 0, text.length, 0, 0);
  };

  whitespace = null;

  setWhitespace = function(ws) {
    return whitespace = ws;
  };

  ParserState = (function() {

    function ParserState(text, pos, end, lineno, xpos) {
      this.text = text;
      this.pos = pos;
      this.end = end;
      this.lineno = lineno;
      this.xpos = xpos;
      this.cache = {};
    }

    ParserState.prototype.advance = function(n) {
      var lineno, pos, state, xpos;
      pos = this.pos;
      lineno = this.lineno;
      xpos = this.xpos;
      while (pos < this.pos + n) {
        if (this.text[pos++] === '\n') {
          lineno++;
          xpos = 0;
        } else {
          xpos++;
        }
      }
      state = new ParserState(this.text, pos, this.end, lineno, xpos);
      state.cache = this.cache;
      return state;
    };

    ParserState.prototype.line = function() {
      var end, lend, lstart, text;
      text = this.text;
      end = this.end;
      lstart = this.pos;
      lend = this.pos;
      if (lstart > 0 && text[lstart] === '\n') {
        lstart--;
      }
      while (lstart > 0 && text[lstart] !== '\n') {
        lstart--;
      }
      if (text[lstart] === '\n') {
        lstart++;
      }
      while (lend < end && text[lend] !== '\n') {
        lend++;
      }
      return text.slice(lstart, lend);
    };

    return ParserState;

  })();

  Match = (function() {

    function Match(state, match) {
      this.state = state;
      this.match = match;
      this.ok = true;
    }

    return Match;

  })();

  NoMatch = (function() {

    function NoMatch(state, message) {
      this.state = state;
      this.message = message;
      this.ok = false;
    }

    return NoMatch;

  })();

  _parser_id = 0;

  Parser = (function() {

    function Parser(message, matcher) {
      this.message = message;
      this.matcher = matcher;
      this.id = _parser_id++;
    }

    Parser.prototype.parse = function(state) {
      var cache, rv;
      if (typeof state === "string") {
        state = newState(state);
      }
      cache = state.cache[this.id];
      if (!(cache != null)) {
        cache = state.cache[this.id] = {};
      }
      rv = cache[state.pos];
      if (rv != null) {
        return rv;
      }
      rv = this.matcher(state);
      cache[state.pos] = rv;
      return rv;
    };

    Parser.prototype.consume = function(state) {
      var rv, rv2;
      rv = this.parse(state);
      if (!rv.ok) {
        return rv;
      }
      rv2 = end.parse(rv.state);
      if (!rv2.ok) {
        return rv2;
      }
      return rv;
    };

    Parser.prototype.fail = function(state) {
      return new NoMatch(state, "Expected " + this.message);
    };

    Parser.prototype.onFail = function(newMessage) {
      var _this = this;
      return new Parser(newMessage, function(state) {
        var rv;
        rv = _this.parse(state);
        if (rv.match != null) {
          return rv;
        }
        return new NoMatch(rv.state, newMessage);
      });
    };

    Parser.prototype.onMatch = function(f) {
      var _this = this;
      return new Parser(this.message, function(state) {
        var rv;
        rv = _this.parse(state);
        if (!rv.ok) {
          return rv;
        }
        if (f instanceof Function) {
          try {
            return new Match(rv.state, f(rv.match));
          } catch (e) {
            return new NoMatch(state, e.toString());
          }
        } else {
          return new Match(rv.state, f);
        }
      });
    };

    Parser.prototype.matchIf = function(f) {
      var _this = this;
      return new Parser(this.message, function(state) {
        var rv;
        rv = _this.parse(state);
        if (!rv.ok) {
          return rv;
        }
        if (!f(rv.match)) {
          return _this.fail(state);
        }
        return rv;
      });
    };

    Parser.prototype.not = function() {
      var _this = this;
      return new Parser("not " + this.message, function(state) {
        var rv;
        rv = _this.parse(state);
        if (rv.ok) {
          return _this.fail(state);
        }
        return new Match(state, "");
      });
    };

    Parser.prototype.or = function() {
      var m, message, others, outer, p, parsers;
      others = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      parsers = (function() {
        var _i, _len, _ref, _results;
        _ref = [this].concat(others);
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          p = _ref[_i];
          _results.push(implicit(p));
        }
        return _results;
      }).call(this);
      message = ((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          m = parsers[_i];
          _results.push(m.message);
        }
        return _results;
      })()).join(" or ");
      outer = this;
      return new Parser(message, function(state) {
        var rv, _i, _len;
        parsers = (function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = parsers.length; _i < _len; _i++) {
            p = parsers[_i];
            _results.push(resolve(p));
          }
          return _results;
        })();
        rv = outer.parse(state);
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          rv = p.parse(state);
          if (rv.ok) {
            return rv;
          }
        }
        return this.fail(state);
      });
    };

    Parser.prototype.skip = function(p) {
      var _this = this;
      p = implicit(p);
      return new Parser(this.message, function(state) {
        var rv;
        p = resolve(p);
        rv = p.parse(state);
        if (rv.ok) {
          state = rv.state;
        }
        return _this.parse(state);
      });
    };

    Parser.prototype.then = function(p) {
      return seq(this, p);
    };

    Parser.prototype.optional = function(defaultValue) {
      if (defaultValue == null) {
        defaultValue = "";
      }
      return optional(this, defaultValue);
    };

    Parser.prototype.repeat = function(sep) {
      if (sep == null) {
        sep = null;
      }
      return repeat(this, sep);
    };

    Parser.prototype.times = function(count) {
      return times(count, this);
    };

    Parser.prototype.reduce = function(sep, f) {
      return foldLeft({
        tail: this,
        accumulator: (function(x) {
          return x;
        }),
        fold: f,
        sep: sep
      });
    };

    Parser.prototype.drop = function() {
      return this.onMatch(function(m) {
        return null;
      });
    };

    Parser.prototype.check = function() {
      var _this = this;
      return new Parser(this.message, function(state) {
        var rv;
        rv = _this.parse(state);
        if (!rv.ok) {
          return rv;
        }
        return new Match(state, rv.match);
      });
    };

    return Parser;

  })();

  end = new Parser("end", function(state) {
    if (state.pos === state.end) {
      return new Match(state, null);
    } else {
      return this.fail(state);
    }
  });

  reject = new Parser("failure", function(state) {
    return this.fail(state);
  });

  string = function(s) {
    var len;
    len = s.length;
    return new Parser("'" + s + "'", function(state) {
      var candidate;
      candidate = state.text.slice(state.pos, state.pos + len);
      if (candidate === s) {
        return new Match(state.advance(len), candidate);
      } else {
        return this.fail(state);
      }
    });
  };

  regex = function(r) {
    var i, m, r2, source;
    i = r.ignoreCase ? "i" : "";
    m = r.multiline ? "m" : "";
    source = r.source[0] === "^" ? r.source : "^" + r.source;
    r2 = new RegExp(source, i + m);
    return new Parser(r.toString(), function(state) {
      m = r2.exec(state.text.slice(state.pos));
      if (m != null) {
        return new Match(state.advance(m[0].length), m);
      } else {
        return this.fail(state);
      }
    });
  };

  seq = function() {
    var p, parsers, ws;
    parsers = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    parsers = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    ws = whitespace;
    return new Parser(parsers[0].message, function(state) {
      var results, rv, _i, _len;
      parsers = (function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push(resolve(p));
        }
        return _results;
      })();
      if (ws != null) {
        parsers = (function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = parsers.length; _i < _len; _i++) {
            p = parsers[_i];
            _results.push(p.skip(ws));
          }
          return _results;
        })();
        ws = null;
      }
      results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        rv = p.parse(state);
        if (!rv.ok) {
          return rv;
        }
        if (rv.match != null) {
          results.push(rv.match);
        }
        state = rv.state;
      }
      return new Match(state, results);
    });
  };

  optional = function(p, defaultValue) {
    if (defaultValue == null) {
      defaultValue = "";
    }
    p = implicit(p);
    return new Parser(p.message, function(state) {
      var rv;
      p = resolve(p);
      rv = p.parse(state);
      if (rv.ok) {
        return rv;
      }
      return new Match(state, defaultValue);
    });
  };

  repeat = function(p, sep) {
    if (sep == null) {
      sep = null;
    }
    return foldLeft({
      tail: p,
      sep: sep
    });
  };

  times = function(count, p) {
    var ws;
    p = implicit(p);
    ws = whitespace;
    return new Parser("" + count + " of (" + p.message + ")", function(state) {
      var i, results, rv, _i;
      p = resolve(p);
      if (ws != null) {
        p = p.skip(ws);
        ws = null;
      }
      results = [];
      for (i = _i = 0; 0 <= count ? _i < count : _i > count; i = 0 <= count ? ++_i : --_i) {
        rv = p.parse(state);
        if (!rv.ok) {
          return this.fail(rv.state);
        }
        if (rv.match != null) {
          results.push(rv.match);
        }
        state = rv.state;
      }
      return new Match(state, results);
    });
  };

  foldLeft = function(args) {
    var accumulator, first, fold, message, sep, tail, ws;
    tail = implicit(args.tail != null ? args.tail : reject);
    first = implicit(args.first != null ? args.first : args.tail);
    fold = args.fold != null ? args.fold : function(acc, s, item) {
      if (item != null) {
        acc.push(item);
      }
      return acc;
    };
    accumulator = args.accumulator != null ? args.accumulator : function(x) {
      if (x != null) {
        return [x];
      } else {
        return [];
      }
    };
    sep = args.sep;
    message = args.first != null ? "(" + first.message + ") followed by (" + tail.message + ")*" : "(" + tail.message + ")*";
    if (sep != null) {
      sep = implicit(sep);
      message += " separated by (" + sep.message + ")";
    }
    ws = whitespace;
    return new Parser(message, function(state) {
      var initial_state, results, rv, sep_match;
      first = resolve(first);
      if (ws != null) {
        first = first.skip(ws);
      }
      if (sep != null) {
        sep = resolve(sep);
        if (ws != null) {
          sep = sep.skip(ws);
        }
      }
      tail = resolve(tail);
      if (ws != null) {
        tail = tail.skip(ws);
      }
      ws = null;
      rv = first.parse(state);
      if (!rv.ok) {
        return this.fail(state);
      }
      results = accumulator(rv.match);
      state = rv.state;
      while (true) {
        initial_state = state;
        sep_match = "";
        if (sep != null) {
          rv = sep.parse(state);
          if (!rv.ok) {
            return new Match(state, results);
          }
          sep_match = rv.match;
          state = rv.state;
        }
        rv = tail.parse(state);
        if (!rv.ok) {
          return new Match(initial_state, results);
        }
        results = fold(results, sep_match, rv.match);
        state = rv.state;
      }
    });
  };

  implicit = function(p) {
    if (typeof p === "string") {
      return string(p);
    }
    if (p instanceof RegExp) {
      return regex(p);
    }
    if (p instanceof Array) {
      return seq.apply(null, p);
    }
    return p;
  };

  resolve = function(p) {
    if (!(p instanceof Function)) {
      return p;
    }
    return implicit(p());
  };

  drop = function(p) {
    return implicit(p).drop();
  };

  parse = function(p, s) {
    return implicit(p).parse(s);
  };

  check = function(p) {
    return implicit(p).check();
  };

  exports.newState = newState;

  exports.setWhitespace = setWhitespace;

  exports.ParserState = ParserState;

  exports.Match = Match;

  exports.NoMatch = NoMatch;

  exports.Parser = Parser;

  exports.end = end;

  exports.reject = reject;

  exports.string = string;

  exports.regex = regex;

  exports.seq = seq;

  exports.optional = optional;

  exports.repeat = repeat;

  exports.times = times;

  exports.foldLeft = foldLeft;

  exports.implicit = implicit;

  exports.drop = drop;

  exports.parse = parse;

  exports.check = check;

}).call(this);
